# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from os import name
if name == 'posix':
    from Amarok import Amarok
    from Exaile import Exaile
    from Xmms import Xmms
    from Banshee import Banshee
    from Audacious import Audacious
    from Rhythmbox import Rhythmbox
    from QuodLibet import QuodLibet
    from Listen import Listen
else:
    from Winamp import Winamp
from CurrentSong import CurrentSong
from Mpd import Mpd

del name

# dont put stuff in the global namespace
